import json
from typing import List

from .Address import Address
from .Email import Email
from .Phone import Phone

class Contact:

    def __init__(
        self,
        display_name: str = '',
        org_name: str = '',
        title: str = '',
        salutation: str = '',
        first_name: str = '',
        middle_name: str = '',
        last_name: str = '',
        name_suffix: str = '',
        nick_name: str = '',
        notes: str = '',
        
        emails: List[Email] = [Email(order=0)],
        phones: List[Phone] = [Phone(order=0)],
        addresses: List[Address] = [Address(order=0)],

        tags: List[str] = []
        ) -> None:
        
        self.display_name = display_name
        self.org_name = org_name
        self.title = title
        self.salutation = salutation
        self.first_name = first_name
        self.middle_name = middle_name
        self.last_name = last_name
        self.name_suffix = name_suffix
        self.nick_name = nick_name
        self.notes = notes
        self.emails = emails
        self.phones = phones
        self.addresses = addresses
        self.tags = tags

    def toDict(self):
        return {
            "display_name": self.display_name,
            "org_name": self.org_name,
            "title": self.title,
            "salutation": self.salutation,
            "first_name": self.first_name,
            "middle_name": self.middle_name,
            "last_name": self.last_name,
            "name_suffix": self.name_suffix,
            "nick_name": self.nick_name,
            "notes": self.notes,
            "emails": [email.toDict() for email in self.emails],
            "phones": [phone.toDict() for phone in self.phones],
            "addresses": [address.toDict() for address in self.addresses],
            "tags": self.tags
        }

    def toJson(self):
        return json.dumps(self.toDict())



import json
from typing import Literal
from xenqu._XenquBase import XenquBase

_prefix = '/jform'

class FormRoutes:

    base: XenquBase

    def __init__(self, base: XenquBase) -> None:
        self.base = base


    """
    * Get form instance data tied to an item on a record

    Parameters
    ----------
        instanceId: int
            ID of a form instance

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#140f933a-dc6d-404a-b072-9af30cb8cf57}
    """
    def getInstance(self, instanceId: int):
        data = self.base.makeGet(f"{_prefix}/instance/{instanceId}")
        return data


    """
    * Lock/Unlock Form

    Parameters
    ----------
        instanceId: int
            ID of a form instance

        action: Literal['lock', 'unlock', 'kick']
            action to perform on form instance

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#7b50fc25-0966-41cd-ad28-47b6b803dec5}
    """
    def setLockStatus(self, instanceId: int, action: Literal['lock', 'unlock', 'kick']):

        if action == 'lock':
            body = {
                "override": True,
                "lock": True
            }
        elif action == 'unlock':
            body = {
                "unlock": True
            }
        else:
            body = {
                "kick": True
            }

        data = self.base.makePut(f"{_prefix}/instance/{instanceId}", payload=json.dumps(body))
        return data


    """
    * Update Field

    Parameters
    ----------
        instanceId: int
            ID of a form instance

        pageId: str
            ID of a specific page on the form

        stateId: str
            ID of the specific field on the form

        value: any
            new value of the field

        fieldId: str
            ID of the specific field on the form

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#fd4f6a78-4cc3-4c34-bc2b-a072b0c4f937}
    """
    def setFieldValue(self, instanceId: int, pageId: str, stateId: str, value: any, fieldId: str):
        body = {
            "raw_value": value,
            "fid": fieldId
        }

        data = self.base.makePut(f"{_prefix}/instance/{instanceId}/page/{pageId}/field/{stateId}", payload=json.dumps(body))
        return data


    """
    * Request a form to be converted to PDF.
    * This is an asynchronous process and require polling the instance for the definition.pdf_id
    * to change values from before the call to this end point to after the call.
    * A 5 second polling interval is recommended.

    Parameters
    ----------
        instanceId: int
            ID of a form instance

        signPdf: bool
            To sign pdf or not to; (that is the question)
        
        addInfoFooter: bool
            To add an info footer or not to; (that is the question)

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#267bf445-2cd4-4a9c-bbf5-d84252193c8e}
    """
    def generatePdf(self, instanceId: int, signPdf: bool, addInfoFooter: bool):
        body = {
            "signed_pdf": signPdf,
            "info_footer": addInfoFooter
        }

        data = self.base.makePost(f"{_prefix}/instance/{instanceId}/pdf/", payload=json.dumps(body))
        return data


    """
    * Files associated with a form can be downloaded by first requesting the file by its file id
    * which will return a temp id to use with the {@link FilesRoutes} 'downloadFile' to download the actual file.
    
    Parameters
    ----------
        instanceId: int
            ID of a form instance

        filesId: str
            ID of the file to retrieve

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#6dd93321-6b80-4b74-af81-b2ccd0328478}
    """
    def getFormFile(self, instanceId: int, filesId: str):
        data = self.base.makeGet(f"{_prefix}/instance/{instanceId}/file/{filesId}")
        return data


from xenqu._XenquBase import XenquBase

_prefix = '/reporting'

class ReportRoutes:

    base: XenquBase

    def __init__(self, base: XenquBase) -> None:
        self.base = base


    """
    * Get Report Results

    Parameters
    ----------
        jobId: str
            job ID for report runs

        status: int
            status of the results to retrieve

        count: int
            number of results to return

        offset: int
            offset of the results

        sortByAsc: bool = True
            sorting of results

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#c5b6a594-36a3-4285-a60c-2580dd0e40a5}
    """
    def getReportResults(self, jobId: str, status: int, count: int, offset: int, sortByAsc: bool = True):
        params = {
            "job_id": jobId,
            "status": status,
            "count": count,
            "offset": offset
        }

        if sortByAsc:
            params["sortby"] = "run_date:asc"
        else:
            params["sortby"] = "run_date:dec"

        data = self.base.makeGet(f"{_prefix}/results/", parameters=params)
        return data


    """
    * Get Report Result

    Parameters
    ----------
        reportId: str
            ID of the specific report to retrieve

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#94d68fc1-e80a-46e5-a4e9-dbef2c562d69}
    """
    def getReportResult(self, reportId: str):
        data = self.base.makeGet(f"{_prefix}/results/{reportId}")
        return data



import json
from xenqu._XenquBase import XenquBase

_prefix = '/files'

class FileRoutes:

    base: XenquBase

    def __init__(self, base: XenquBase) -> None:
        self.base = base


    """
    * Download File

    Parameters
    ----------
        tempHandleId: str
            temporary ID that represents the file to download

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#2102ba3f-8a86-4f02-aeb2-3cdfbda3eadc}
    """
    def download(self, tempHandleId: str):
        data = self.base.makeGet(f"{_prefix}/{tempHandleId}", bytes=True)
        return data


    """
    * File uploads can be chunked or sent in one large chunk. Total file size can be 50Mb
    * fileHandle in the response represents the _temp_handle_id sent to other end points used to attach files to an object.
    * Uploads in predefined chunk sizes asynchronously for fast uploads
    
    Parameters
    ----------
        chunkData: str
        chunkSeq: int
        chunkStart: int
        chunkEnd: int
        chunkSize: int
        chunkLimit: int
        totalSize: int
        totalChunks: int

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#aa314a75-2acb-42f9-9ff4-ad10282ee961}
    """
    def upload(self, chunkData: str, chunkSeq: int, chunkStart: int, chunkEnd: int, chunkSize: int, chunkLimit: int, totalSize: int, totalChunks: int):
        body = {
            "chunkData": chunkData,
            "chunkSeq": chunkSeq,
            "chunkStart": chunkStart,
            "chuckEnd": chunkEnd,
            "chunkSize": chunkSize,
            "chunkLimit": chunkLimit,
            "totalSize": totalSize,
            "totalChunks": totalChunks
        }

        data = self.base.makePost(f"{_prefix}", payload=json.dumps(body))
        return data



import time
import requests
import jwt
from jwcrypto import jwk, jwt
from requests_oauthlib import OAuth1
from xenqu.models import OAuth2Token

class XenquBase:

    def __init__(self, baseUrl: str, clientId: str = '', clientSecret: str = '') -> None:
        self.baseUrl = baseUrl
        self.clientId = clientId
        self.clientSecret = clientSecret

        self.apiUrl = self.baseUrl + '/api'
        self.oauth = OAuth2Token()


    """
    * Update Oath Paramaters
    
    Parameters
    ----------
        oauth: OAuth2Token
            Oauth Token Information
    """
    def updateOauth(self, oauth: OAuth2Token):
        self.oauth = oauth


    """
    * Perform {GET} request

    Parameters
    ----------
        path: str
            suffix url for api call

        paramaters: dict
            inline request parameters
    """
    def makeGet(self, path: str, parameters: dict = None, bytes: bool = False):
        url = self.apiUrl + path

        oauth = OAuth1(client_key=self.clientId, client_secret=self.clientSecret, resource_owner_key=self.oauth.token, resource_owner_secret=self.oauth.tokenSecret)
        res = requests.get(url=url, params=parameters, auth=oauth)

        if res.status_code in range(200, 211):
            if bytes:
                return res.content
            else:
                return res.text
        else:
            raise Exception(f"Error at OAuth 2.0 Endpoint: {res.content}\nURL: {res.url}\nBody: {res.request.body}\nStatus Code: {res.status_code}")


    """
    * Perform {POST} request

    Parameters
    ----------
        path: str
            suffix url for api call

        payload: str
            body of the request

        paramaters: dict
            inline request parameters
    """
    def makePost(self, path: str, payload: str = '{}'):
        url = self.apiUrl + path

        oauth = OAuth1(client_key=self.clientId, client_secret=self.clientSecret, resource_owner_key=self.oauth.token, resource_owner_secret=self.oauth.tokenSecret)
        res = requests.post(url=url, data=payload, headers={"Content-Type": "application/json"}, auth=oauth)

        if res.status_code > 199 and res.status_code < 210:
            return res.text
        else:
            raise Exception(f"Error at OAuth 2.0 Endpoint: {res.content}\nURL: {res.url}\nBody: {res.request.body}\nStatus Code: {res.status_code}")


    """
    * Perform {PUT} request

    Parameters
    ----------
        path: str
            suffix url for api call

        payload: str
            body of the request

        paramaters: dict
            inline request parameters
    """
    def makePut(self, path: str, payload: str):
        url = self.apiUrl + path

        oauth = OAuth1(client_key=self.clientId, client_secret=self.clientSecret, resource_owner_key=self.oauth.token, resource_owner_secret=self.oauth.tokenSecret)
        res = requests.put(url=url, data=payload, headers={"Content-Type": "application/json"}, auth=oauth)

        if res.status_code > 199 and res.status_code < 210:
            print(f"Body: {res.request.body}")
            return res.text
        else:
            raise Exception(f"Error at OAuth 2.0 Endpoint: {res.content}\nURL: {res.url}\nBody: {res.request.body}\nStatus Code: {res.status_code}")


    """
    * Perform {DELETE} request

    Parameters
    ----------
        path: str
            suffix url for api call

        payload: str
            body of the request

        paramaters: dict
            inline request parameters
    """
    def makeDelete(self, path: str, payload: str = None, paramaters: dict = None):
        url = self.apiUrl + path

        oauth = OAuth1(client_key=self.clientId, client_secret=self.clientSecret, resource_owner_key=self.oauth.token, resource_owner_secret=self.oauth.tokenSecret)
        res = requests.delete(url=url, data=payload, headers={"Content-Type": "application/json"}, auth=oauth)

        if res.status_code > 199 and res.status_code < 210:
            return res.text
        else:
            raise Exception(f"Error at OAuth 2.0 Endpoint: {res.content}\nURL: {res.url}\nBody: {res.request.body}\nStatus Code: {res.status_code}")


    """
    * Does the Oauth stuff

    Parameters
    ----------
        pemPrivateKey: bytes
            raw bytes from reading the pem file
    """
    def makeOauth2Request(self, pemPrivateKey: bytes, subscriber: str) -> OAuth2Token:
        key = jwk.JWK.from_pem(pemPrivateKey)
        payload = {
            "exp": time.time() + 300,
            "iss": self.clientId,
            "aud": self.baseUrl,
            "sub": subscriber
        }

        raw_jwt_token = jwt.JWT(header={"alg": "RS256"}, claims=payload)
        raw_jwt_token.make_signed_token(key)

        signed_token = raw_jwt_token.serialize()

        payload_data = f'grant_type=urn%3Aietf%3Aparams%3Aoauth%3Agrant-type%3Ajwt-bearer&assertion={signed_token}'

        oauth_url = self.apiUrl + '/oauth2/token'
        res = requests.post(url=oauth_url, data=payload_data, auth=(self.clientId, self.clientSecret))

        if res.status_code in range(200, 211):
            return OAuth2Token(res.text)
        else:
            raise Exception(f"## Error with OAuth 2.0 Request ##\n\tStatus Code: {res.status_code}\n\tResponse: {res.text}")



from xenqu._XenquBase import XenquBase
from xenqu.routes.Tracking import _prefix

class RecordRoutes:

    base: XenquBase

    def __init__(self, base: XenquBase) -> None:
        self.base = base


    """
    * Get a record and its content on a specific tab

    Parameters
    ----------
        groupId: int

        contactId: int

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#3bd6f73c-b25e-4d3f-8695-5e58d35df88d}
    """
    def getRecord(self, groupId: int, contactId: int):
        data = self.base.makeGet(f"{_prefix}/groups/{groupId}/actors/{contactId}")
        return data


    """
    * Initiates running automation rules on the record. This should be called after saving all queues or
    * editing any data that does not result in an item action (changing status, actors,etc)
    
    Parameters
    ----------
        groupId: int
        
        contactId: int

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#f7b392bb-169b-4aca-b8d9-9cbafc471f9a}
    """
    def applyRules(self, groupId: int, contactId: int):
        data = self.base.makePost(f"{_prefix}/groups/{groupId}/actors/{contactId}/apply_rules")
        return data



import json
from typing import List

from .Item import Item

class Queue:

    def __init__(
        self,
        title: str = '',
        primary_actor_role_slug: str = 'worker',
        primary_actor_role: str = 'Worker',
        primary_actor_id: int = -1,
        section: int = 1,
        force_ordering: int = 0,
        progress_bin_id: int = None,

        items: List[Item] = [Item()]
        ) -> None:
        
        self.title = title
        self.primary_actor_role_slug = primary_actor_role_slug
        self.primary_actor_role = primary_actor_role
        self.primary_actor_id = primary_actor_id
        self.section = section
        self.force_ordering = force_ordering
        self.progress_bin_id = progress_bin_id
        self.items = items

    def toDict(self):
        return {
            "title": self.title,
            "primary_actor_role_slug": self.primary_actor_role_slug,
            "primary_actor_role": self.primary_actor_role,
            "primary_actor_id": self.primary_actor_id,
            "section": self.section,
            "force_ordering": self.force_ordering,
            "progress_bin_id": self.progress_bin_id,
            "items": [item.toDict() for item in self.items]
        }

    def toJson(self):
        return json.dumps(self.toDict())


import json
from typing import List
from xenqu._XenquBase import XenquBase

from xenqu.routes.Tracking import _prefix

class TabRoutes:

    base: XenquBase

    def __init__(self, base: XenquBase) -> None:
        self.base = base


    """
    * Retrieve all contacts the current user can access within the scope of their tab membership, contact ownership, and task assignments
    
    Parameters
    ----------
        term: str
            search term for contact

        scope: List[str]
            search term scope

        count: int
            number of contacts to retrieve

        offset: int
            results offset

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#7dab4692-f5f6-420d-9f46-dfc10c55e7d0}
    """
    def getContacts(self, term: str, scope: List[str], count: int, offset: int):
        params = {
            "term": term,
            "scope": scope,
            "count": count,
            "offset": offset
        }

        data = self.base.makeGet(f"{_prefix}/contacts", parameters=params)
        return data


    """
    * Get a list of tabs the current user is a member of

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#764d0965-219e-41a0-b0cc-5896c39012c8}
    """
    def getTabs(self):
        data = self.base.makeGet(f"{_prefix}/groups")
        return data


    """
    * Get a list of all available items for a given tab that can be added to a record
    
    Parameters
    ----------
        groupId: int

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#f89c6d8b-b7f6-48a2-9ab7-5ffc0029b326}
    """
    def getItems(self, groupId: int):
        data = self.base.makeGet(f"{_prefix}/groups/{groupId}/list/items")
        return data


    """
    * Get a list of predefined templates that can used to add content to a record

    Parameters
    ----------
        groupId: int

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#c1386fb4-46fb-4934-91c4-e811f0fb56de}
    """
    def getQueueTemplates(self, groupId: int):
        data = self.base.makeGet(f"{_prefix}/groups/{groupId}/queue_templates")
        return data


    """
    * Load the predefined queues and items based on the template id
    
    Parameters
    ----------
        groupId: int

        templateId: str

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#76dd3859-1f6f-479d-bd58-64b742cf8bdf}
    """
    def getQueueTemplate(self, groupId: int, templateId: str):
        data = self.base.makeGet(f"{_prefix}/groups/{groupId}/queue_templates/{templateId}")
        return data


    """
    * Get a list of all content libraries available on a tab
    
    Parameters
    ----------
        groupId: int

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#2d7ae85c-9ea3-4832-83f5-14f982756c20}
    """
    def getLibraries(self, groupId: int):
        data = self.base.makeGet(f"{_prefix}/groups/{groupId}/libraries")
        return data


    """
    * Search for items and/or libraries by title

    Parameters
    ----------
        groupId: int

        title: str

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#ee9ddc77-a8a6-49b7-a7d7-f9a72c44d00c}
    """
    def searchLibraries(self, groupId: int, title: str):
        body = {
            "title": title
        }

        data = self.base.makePost(f"{_prefix}/groups/{groupId}/libraries/select", payload=json.dumps(body))
        return data



from typing import Any, Dict

import json
import base64
import requests
from requests.models import Response



def b64EncodeBytes(b: bytes) -> str:
    return base64.b64encode(b).decode('ascii')

def performGet(url: str, headers: Dict[str, Any]) -> Response:
    return requests.get(url=url, headers=headers)

def performPost(url: str, headers: Dict[str, Any], jsonData: dict) -> Response:
    return requests.post(url=url, headers=headers, json=json.loads(jsonData))

def performPut(url: str, headers: Dict[str, Any], jsonData: dict) -> Response:
    return requests.put(url=url, headers=headers, json=json.loads(jsonData))

def performDelete(url: str, headers: Dict[str, Any], jsonData: dict) -> Response:
    return requests.delete(url=url, headers=headers, json=json.loads(jsonData))


import json

class Actor:

    def __init__(
        self,
        order: int = 0,
        contact_id: int = 0,
        actor_role: str = 'Worker',
        actor_role_slug: str = 'worker',
        status: str = None
        ) -> None:

        self.order = order
        self.contact_id = contact_id
        self.actor_role = actor_role
        self.actor_role_slug = actor_role_slug
        self.status = status

    def toDict(self):
        return {
            "order": self.order,
            "contact_id": self.contact_id,
            "actor_role": self.actor_role,
            "actor_role_slug": self.actor_role_slug,
            "status": self.status
        }

    def toJson(self):
        return json.dumps(self.toDict())



from .AccountRoutes import AccountRoutes
from .ContactRoutes import ContactRoutes
from .FileRoutes import FileRoutes
from .FormRoutes import FormRoutes
from .ReportRoutes import ReportRoutes
from .SearchRoutes import SearchRoutes
from .TrackingRoutes import TrackingRoutes


import json
from typing import Literal
from xenqu._XenquBase import XenquBase
from xenqu.models import TrackingItem

from xenqu.routes.Tracking import _prefix

class ItemRoutes:

    base: XenquBase

    def __init__(self, base: XenquBase) -> None:
        self.base = base


    """
    * Get list of items for current user. This data is the same as seen under the "My Queue" menu in the UI. queued now complete

    Parameters
    ----------
        queued: Literal["open", "closed"] = "open"
            item queue status

        sortBy: Literal["relevance", "create_date", "last_log_date", "schedule_date", "priority", "item_title"] = "last_log_date"
            sort options

        descending: bool = True
            sort order

        offset: int = 0
            sort offset

        count: int = 20
            number of results to retrieve

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#96bd981d-1505-4353-86f7-35f17d5a2f69}
    """
    def getAssignedItemList(self, queued: Literal["open", "closed"] = "open", sortBy: Literal["relevance", "create_date", "last_log_date", "schedule_date", "priority", "item_title"] = "last_log_date", descending: bool = True, offset: int = 0, count: int = 20):

        params = {
            "queued": queued,
            "sortby": sortBy,
            "offset": offset,
            "count": count
        }

        if descending:
            params["sortby"] = params["sortby"] + ',desc'
            data = self.base.makeGet(f"{_prefix}/user/items", parameters=params)

        else:
            data = self.base.makeGet(f"{_prefix}/user/items", parameters=params)

        return data


    """
    * Get more details about an item assigned to the current user. Includes the complete activity log on the item.

    Parameters
    ----------
        trackingId: int
            tracking ID to retrieve details from

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#c909077e-1ac7-4501-9b0b-626d8bd75694}
    """
    def getAssignedItemDetail(self, trackingId: int):
        data = self.base.makeGet(f"{_prefix}/user/items/{trackingId}")
        return data


    """
    * Get detail about a tracking item on a record

    Parameters
    ----------
        groupId: int

        queueId: int

        trackingId: int

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#9b97b262-1934-449f-9660-81770f6385f3}
    """
    def getTrackingItem(self, groupId: int, queueId: int, trackingId: int):
        data = self.base.makeGet(f"{_prefix}/groups/{groupId}/queues/{queueId}/items/{trackingId}")
        return data


    """
    * Update a tracking item on a record

    Parameters
    ----------
        groupId: int

        queueId: int

        trackingId: int

        trackingItem: TrackingItem

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#16602514-0065-4db4-8327-1c15338579fb}
    """
    def updateTrackingItem(self, groupId: int, queueId: int, trackingId: int, trackingItem: TrackingItem):
        data = self.base.makePut(f"{_prefix}/groups/{groupId}/queues/{queueId}/items/{trackingId}", payload=trackingItem.toJson())
        return data


    """
    * This will post a message on the item which is visible to all actors in their My Queue view,
    * on the dashboard, and, if subscribed, email/text to each assigned actor.
    
    Parameters
    ----------
        trackingId: int
            tracking ID of the item

        messageText: str
            message to add to item

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#e3c14915-4491-4b37-a381-0af2d2ef10ce}
    """
    def addMessage(self, trackingId: int, messageText: str):
        body = {
            "event_data": {
                "message_text": messageText
            }
        }
        
        data = self.base.makePost(f"{_prefix}/items/{trackingId}/logs", payload=json.dumps(body))
        return data


    """
    * Set a callback with a url to post back against when anything changes on the given item.
    
    Parameters
    ----------
        groupId: int

        queueId: int

        trackingId: int

        callbackUrl: str

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#bda9a19c-f79f-466e-b47b-b230bda2b3e4}
    """
    def setCallback(self, groupId: int, queueId: int, trackingId: int, callbackUrl: str):
        body = {
            "callback_url": callbackUrl
        }

        data = self.base.makePut(f"{_prefix}/groups/{groupId}/queues/{queueId}/items/{trackingId}/callback", payload=json.dumps(body))
        return data


    """
    * Remove callback previously set on an item

    Parameters
    ----------
        groupId: int

        queueId: int

        trackingId: int

        callbackUrl: str

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#2ce6d984-d018-464d-a024-a116f04d9e05}
    """
    def unsetCallback(self, groupId: int, queueId: int, trackingId: int, callbackUrl: str):
        body = {
            "callback_url": callbackUrl
        }

        data = self.base.makeDelete(f"{_prefix}/groups/{groupId}/queues/{queueId}/items/{trackingId}/callback", payload=json.dumps(body))
        return data


    """
    * By tab, get a list of items ordered by most recent activity to oldest

    Parameters
    ----------
        groupId: int

        count: int
            number of items to retrieve

        offset: int
            data offset

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#b0730376-afa6-4831-806d-5c54411e6ff7}
    """
    def getActivityStream(self, groupId: int, count: int, offset: int):
        params = {
            "count": count,
            "offset": offset
        }

        data = self.base.makeGet(f"{_prefix}/groups/{groupId}/items/activity", parameters=params)
        return data


    """
    * For tracking items of type "attachment", list all the files associated with the item

    Parameters
    ----------
        trackingId: int
            tracking ID of the item to get attachments from

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#cd32010b-15e7-4287-aa8d-4e81645eec3d}
    """
    def getListAttachments(self, trackingId: int):
        data = self.base.makeGet(f"{_prefix}/attachments/{trackingId}")
        return data


    """
    * For tracking items of type "attachment", list all the files associated with the item

    Parameters
    ----------
        attachmentId: str
            ID of the attachment to download

        filesId: str

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#cd32010b-15e7-4287-aa8d-4e81645eec3d}
    """
    def getDownloadAttachment(self, attachmentId: str, filesId: str):
        data = self.base.makeGet(f"{_prefix}/attachments/{attachmentId}/files/{filesId}")
        return data


    """
    * For tracking items of type "attachment", list all the files associated with the item

    Parameters
    ----------
        trackingId: int

        tempHandleId: str

        attachmentId: str

        contentType: str

        fileName: str

        order: str

        filesId: str

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#db3c24c6-a6d7-4f44-8eb3-9a82b70753e3}
    """
    def addAttachment(self, trackingId: int, tempHandleId: str, attachmentId: str, contentType: str, fileName: str, order: int, filesId: str = None):
        body = {
            "tracking_id": trackingId,
            "attachment_id": attachmentId,
            "files_id": filesId,
            "content_type": contentType,
            "filename": fileName,
            "order": order,
            "_temp_handle_id": tempHandleId
        }

        data = self.base.makePost(f"{_prefix}/attachments/{attachmentId}/files", payload=json.dumps(body))
        return data


    """
    * For tracking items of type "attachment", list all the files associated with the item
    
    Parameters
    ----------
        trackingId: int

        attachmentId: str

        filesId: str

        contentType: str

        fileName: str

        order: int

        tempHandleId: str

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#0b52cc8c-40e4-40d6-8b95-266f7c8f139d}
    """
    def updateAttachment(self, trackingId: int, attachmentId: str, filesId: str, contentType: str, fileName: str, order: int, tempHandleId: str):
        body = {
            "tracking_id": trackingId,
            "attachment_id": attachmentId,
            "files_id": filesId,
            "content_type": contentType,
            "filename": fileName,
            "order": order,
            "_temp_handle_id": tempHandleId
        }

        data = self.base.makePut(f"{_prefix}/attachments/{attachmentId}/files/{filesId}", payload=json.dumps(body))
        return data


    """
    * For tracking items of type "attachment", list all the files associated with the item
    
    Parameters
    ----------
        attachmentId: str
            ID of the attachment to delete

        filesId: str

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#2026a2a1-5455-4e60-a782-649b8101b7e2}
    """
    def deleteAttachment(self, attachmentId: str, filesId: str):
        data = self.base.makeDelete(f"{_prefix}/attachments/{attachmentId}/files/{filesId}")
        return data

    


import json
import datetime

from typing import List, Literal, Optional, Union

from .Actor import Actor

class TrackingItem:

    def _is_valid_extra_label(self, s: str):
        if len(s) > 35:
            raise ValueError("The Extra Label property must not be longer than 35 characters.")
        return s

    def __init__(
        self,
        status: Literal["open", "closed", "skip"] = "open",
        schedule_date: Union[Literal[None], datetime.date] = None,
        priority: Literal[1, 2, 3] = 2,
        extra_label: str = '',
        
        actors: Optional[List[Actor]] = [Actor()]
        ) -> None:
        
        self.status = status
        self.schedule_date = schedule_date
        self.priority = priority
        self.extra_label = self._is_valid_extra_label(extra_label)
        self.actors = actors


    def toDict(self):
        return {
            "status": self.status,
            "schedule_date": self.schedule_date,
            "priority": self.priority,
            "extra_label": self.extra_label,
            "item": {
                "actors": [actor.toDict() for actor in self.actors] if self.actors != None else None
            }
        }

    def toJson(self):
        return json.dumps(self.toDict())



from xenqu._XenquBase import XenquBase
from xenqu.routes.Tracking import TabRoutes, RecordRoutes, QueueRoutes, ItemRoutes

class TrackingRoutes:

    def __init__(self, base: XenquBase) -> None:
        self.base = base

        self.tabs = TabRoutes(self.base)
        self.records = RecordRoutes(self.base)
        self.queues = QueueRoutes(self.base)
        self.items = ItemRoutes(self.base)

    def update(self):
        self.tabs.base = self.base
        self.records.base = self.base
        self.queues.base = self.base
        self.items.base = self.base


from xenqu._XenquBase import XenquBase
from xenqu.models import Queue
from xenqu.routes.Tracking import _prefix

class QueueRoutes:

    base: XenquBase

    def __init__(self, base: XenquBase) -> None:
        self.base = base


    """
    * Get the latest values for a queue including all its items.
    
    Parameters
    ----------
        groupId: int

        queueId: int

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#bc053d06-4bbd-4c47-9c09-7db3454d16bd}
    """
    def getQueue(self, groupId: int, queueId: int):
        data = self.base.makeGet(f"{_prefix}/groups/{groupId}/queues/{queueId}")
        return data


    """
    * Create a queue

    Parameters
    ----------
        groupId: int

        queue: Queue
            queue to add

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#38e912b5-5c85-47f7-9800-299c5d66e773}
    """
    def addQueue(self, groupId: int, queue: Queue):
        data = self.base.makePost(f"{_prefix}/groups/{groupId}/queues", payload=queue.toJson())
        return data


    """
    * Edit a queue

    Parameters
    ----------
        groupId: int

        queueId: int
            ID of the queue to modify

        queue: Queue
            new queue data

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#9c87cc46-1abb-4c76-a973-4ad3a8e5aec9}
    """
    def editQueue(self, groupId: int, queueId: int, queue: Queue):
        data = self.base.makePut(f"{_prefix}/groups/{groupId}/queues/{queueId}", payload=queue.toJson())
        return data


    """
    * Delete a queue
    
    Parameters
    ----------
        groupId: int

        queueId: int

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#de5d65c9-0d96-4b7a-8436-f0a450b21e5f}
    """
    def deleteQueue(self, groupId: int, queueId: int):
        data = self.base.makeDelete(f"{_prefix}/groups/{groupId}/queues/{queueId}")
        return data


import json
from typing import List

from .Actor import Actor

class Item:

    def __init__(
        self,
        order: int = 0,
        tracking_id: int = 1,
        item_id: int = -1,
        tracking_library_id: int = -1,
        
        actors: List[Actor] = []
        ) -> None:
        
        self.order = order
        self.tracking_id = tracking_id
        self.item_id = item_id
        self.tracking_library_id = tracking_library_id
        self.actors = actors

    def toDict(self):
        return {
            "order": self.order,
            "tracking_id": self.tracking_id,
            "item_id": self.item_id,
            "tracking_library_id": self.tracking_library_id,
            "item": {
                "actors": [actor.toDict() for actor in self.actors]
            }
        }

    def toJson(self):
        return json.dumps(self.toDict())




import json

class Address:

    def __init__(
        self,
        order: int = 0,
        street1: str = '',
        street2: str = '',
        street3: str = '',
        region1: str = '',
        region2: str = '',
        postalcode: str = '',
        country: str = '',
        valid_from: str = '',
        valid_to: str = '',
        ) -> None:
        
        self.order = order
        self.street1 = street1
        self.street2 = street2
        self.street3 = street3
        self.region1 = region1
        self.region2 = region2
        self.postalcode = postalcode
        self.country = country
        self.valid_from = valid_from
        self.valid_to = valid_to

    def toDict(self):
        return {
            "order": self.order,
            "usage": "Address",
            "street1": self.street1,
            "street2": self.street2,
            "street3": self.street3,
            "region1": self.region1,
            "region2": self.region2,
            "postalcode": self.postalcode,
            "country": self.country,
            "valid_from": self.valid_from,
            "valid_to": self.valid_to
        }

    def toJson(self):
        return json.dumps(self.toDict())


from .Actor import Actor
from .Address import Address
from .Contact import Contact
from .Email import Email
from .Item import Item
from .OAuth2Token import OAuth2Token
from .Phone import Phone
from .Queue import Queue
from .TrackingItem import TrackingItem



import json
from datetime import datetime
from typing import Any, OrderedDict

class OAuth2Token:

    def __init__(self, jsonData: str = None) -> None:
        self.token: str = ''
        self.tokenSecret: str = ''
        self.expires = datetime(year=1970, month=1, day=1, hour=0, minute=0, second=0)

        if jsonData:
            try:
                self.fromJson(json.loads(jsonData))
            except:
                raise Exception("## Invalid JSON format ##")

    def fromJson(self, json: OrderedDict[str, Any]):
        self.token = json["token"]
        self.tokenSecret = json["token_secret"]
        self.expires = datetime.fromtimestamp(json["expires"])


import setuptools

setuptools.setup(
    name="xenqu-api",
    version="1.0.4",
    author="Charles Buffington II",
    author_email="charles.buffington@essium.co",
    description="A full implementation of the Xenqu API in Python.",
    url="https://github.com/EssiumLLC/lib-xenqu-python",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=['requests', 'jwcrypto', 'requests_oauthlib']
)


import json

class Phone:

    def __init__(
        self,
        order: int = 0,
        format: str = '',
        number: str = ''
        ) -> None:
        
        self.order = order
        self.format = format
        self.number = number

    def toDict(self):
        return {
            "order": self.order,
            "usage": "Phone",
            "format": self.format,
            "number": self.number
        }

    def toJson(self):
        return json.dumps(self.toDict())


import json

class Email:

    def __init__(
        self,
        order: int = 0,
        address: str = ''
        ) -> None:
        
        self.order = order
        self.address = address

    def toDict(self):
        return {
            "order": self.order,
            "usage": "Email",
            "address": self.address
        }

    def toJson(self):
        return json.dumps(self.toDict())



from xenqu._XenquBase import XenquBase
from xenqu.models import Contact

_prefix = '/contact'

class ContactRoutes:

    base: XenquBase

    def __init__(self, base: XenquBase) -> None:
        self.base = base


    """
    * The preferred end point for accessing contacts is tracking/contacts.
    * This end point only returns contacts created by the current user.
    * Refer to that documentation for options.

    Parameters
    ----------
        term: str
            search term
        
    ? @see [API Docs]{@link https://apidocs.xenqu.com/#e0fbbe9f-1ae3-4523-adff-ea8506ce7e0a}
    """
    def searchContacts(self, term: str):
        data = self.base.makeGet(f"{_prefix}?term={term}")
        return data


    """
    * Get full details on a contact

    Parameters
    ----------
        contactId: int
            ID of the contact to retrieve

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#24bb22da-7e96-4c16-8c32-c4bca1aeed86}
    """
    def getContactDetail(self, contactId: int):
        data = self.base.makeGet(f"{_prefix}/{contactId}")
        return data


    """
    * Add a new contact. Returns created contact.

    Parameters
    ----------
        contact: Contact
            contact data to add

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#7969a360-ee09-41a8-9299-05a5f0ffb7fb}
    """
    def addContact(self, contact: Contact):
        data = self.base.makePost(f"{_prefix}", payload=contact.toJson())
        return data


    """
    * Edit a contact. Returns edited contact.

    Parameters
    ----------
        contactId: int
            ID of the contact to edit

        contact: Contact
            contact data to modify

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#7ef9b643-8086-4ca0-ad5c-05895dc3eb52}
    """
    def editContact(self, contactId: int, contact: Contact):
        data = self.base.makePut(f"{_prefix}/{contactId}", payload=contact.toJson())
        return data


    """
    * Delete a contact. Returns deleted contact id.

    Parameters
    ----------
        contactId: int
            ID of the contact to delete

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#027b6abb-2ac1-40dc-a82c-afe4a631aae3}
    """
    def deleteContact(self, contactId: int):
        data = self.base.makeDelete(f"{_prefix}/{contactId}")
        return data



from xenqu._XenquBase import XenquBase
from xenqu.routes import AccountRoutes, ContactRoutes, TrackingRoutes, FormRoutes, ReportRoutes, FileRoutes, SearchRoutes

class XenquAPI:
    
    def __init__(
        self,
        clientId: str, 
        clientSecret: str, 
        pemPrivateKey: bytes, 
        subscriber: str, 
        baseUrl: str = 'https://xenqu.com/api') -> None:
        
        self.clientId = clientId
        self.clientSecret = clientSecret
        self.pemPrivateKey = pemPrivateKey
        self.subscriber = subscriber
        self.baseUrl = baseUrl

        self.base = XenquBase(baseUrl=self.baseUrl, clientId=self.clientId, clientSecret=self.clientSecret)
        self.isInit: bool = False

        self.account = AccountRoutes(self.base)
        self.contact = ContactRoutes(self.base)
        self.tracking = TrackingRoutes(self.base)
        self.forms = FormRoutes(self.base)
        self.reports = ReportRoutes(self.base)
        self.files = FileRoutes(self.base)
        self.search = SearchRoutes(self.base)

        self.init()

    def init(self) -> None:
        if not self.isInit:
            return self.reauth()
        else:
            raise Exception("Already Initiated")

    def reauth(self) -> None:
        self.isInit = False
        token = self.base.makeOauth2Request(self.pemPrivateKey, self.subscriber)
        self.base.updateOauth(token)
        self._updateRoutes()
        self.isInit = True

    def _updateRoutes(self) -> None:
        self.account.base = self.base
        self.contact.base = self.base
        
        self.tracking.base = self.base
        self.tracking.update()

        self.forms.base = self.base
        self.reports.base = self.base
        self.files.base = self.base
        self.search.base = self.base


from xenqu._XenquBase import XenquBase

_prefix = '/user'

class AccountRoutes:

    base: XenquBase

    def __init__(self, base: XenquBase) -> None:
        self.base = base


    """
    * Gets the Current API User's Information

    ? @see [API Docs]{@link https://apidocs.xenqu.com/#3e68223a-c6a6-4469-b2be-5a43315c7210}
    """
    def getUserInfo(self):
        data = self.base.makeGet(f'{_prefix}/info')
        return data